{% extends "base.html" %}

{% block title %}
	&raquo; Meter Payment Types
{% endblock %}

{% block nav %}
	Meter Payment Types
{% endblock %}

{% block content %}

	<table>
		<caption>Meter Payment Types</caption>	
		<thead>
			<tr>
				<th>Code</th>
				<th>Description</th>
				<th>Valid From</th>
				<th>Valid To</th>
			</tr>
		</thead>
		<tbody>
			{% for meter_payment_type in meter_payment_types %}
				<tr>
					<td>
						<a href="/e/meter_payment_types/{{meter_payment_type.id}}"
							>{{meter_payment_type.code}}</a>
					</td>
					<td>{{ meter_payment_type.description }}</td>
					<td>{{meter_payment_type.valid_from|hh_format}}</td>
					<td>{{meter_payment_type.valid_to|hh_format}}</td>
				</tr>
			{% endfor %}
		</tbody>
	</table>

{% endblock %}
