{% extends "base.html" %}

{% block title %}
	&raquo; Reading Frequencies
{% endblock %}

{% block nav %}
	Reading Frequencies
{% endblock %}

{% block content %}
	<table>
		<thead>
			<tr>
				<th>Code</th>
				<th>Description</th>
			</tr>
		</thead>
		<tbody>
			{% for g_reading_frequency in g_reading_frequencies %}
				<tr>
					<td>
						<a href="/g_reading_frequencies/{{g_reading_frequency.id}}">
							{{g_reading_frequency.code}}
						</a>
					</td>
					<td>{{g_reading_frequency.description}}</td>
				</tr>
			{% endfor %}
		</tbody>
	</table>
{% endblock %}
