{% extends "base.html" %}

{% block title %}
	&raquo; TPRs
{% endblock %}

{% block nav %}
	TPRs
{% endblock %}

{% block content %}
	<ul>
		{% for tpr in tprs %}
			<li><a href="/e/tprs/{{tpr.id}}">{{tpr.code}}</a></li>
		{% endfor %}
	</ul>
{% endblock %}
